import java.util.concurrent.ThreadLocalRandom;
import java.util.Arrays;

class Exo4 {

    static final int nbCells = 1_000_000;
    static final int nIterationsDefault = 100;
    static final int ruleIDDefault = 110;
    static int N, nIterations, ruleID;
    static int []seqOutput;
    static int []par2Output;
    static int []parNOutput;
    public static void main(String []argv) {

        N = getNbThreads(argv);
        nIterations = getNIterations(argv);
        ruleID = getRuleID(argv);
        int []initConfiguration = randomConfiguration();
        float seqTime = Chrono.run( () -> {
            seqOutput = nthConfig(initConfiguration, nIterations);
        }); 
        System.out.printf("Temps avec nthConfig: %.2f\n", seqTime);
        float par2Time = Chrono.run( () -> {
            par2Output = nthConfigRecycling(initConfiguration, nIterations);
        }); 
        System.out.printf("Résultat correct avec nthConfigRecycling : %b\n", Arrays.equals(par2Output, seqOutput));
        System.out.printf("Speedup avec nthConfigRecycling: %.2f\n", seqTime / par2Time);
        float parNTime = Chrono.run( () -> {
            parNOutput  = nthConfigParallelN(initConfiguration, nIterations);
        }); 
        System.out.printf("Résultat correct avec nthConfigParallelN : %b\n", Arrays.equals(parNOutput, seqOutput));
        System.out.printf("Speedup avec nthConfigParallelN (N=%d): %.2f\n", N, seqTime / parNTime);
    }

    static int localRule(int left, int center, int right){
        int x = left + 2 * center + 4 * right;
        return (ruleID & (1 << x)) >> x;
    }

    static int getNbThreads(String []argv){
        try { return Integer.parseInt(argv[0]); }
        catch (Exception e) { return Runtime.getRuntime().availableProcessors();}
    }

    static int getNIterations(String []argv){
        try { return Integer.parseInt(argv[1]); }
        catch (Exception e) { return nIterationsDefault; }
    }

    static int getRuleID(String []argv){
        try { return Integer.parseInt(argv[2]); }
        catch (Exception e) { return ruleIDDefault; }
    }

    static int []nextConfig(int []conf){
        int []res = new int[nbCells];
        for(int i=0; i<nbCells; i++){
            res[i] = localRule(conf[(nbCells + i - 1) % nbCells], conf[i], conf[(i+1) % nbCells]);
        }
        return res;
    }

    static int []nthConfig(int []configuration, int n){
        for(int i=0; i<n; i++){
            configuration = nextConfig(configuration);
        }
        return configuration;
    }

    static int []nthConfigRecycling(int []configuration, int n){
        // A modifier!
        return nthConfig(configuration, n);
    }

    static int []nthConfigParallelN(int []configuration, int n){
        // A modifier!
        return nthConfig(configuration, n);
    }

    static int []randomConfiguration(){
        ThreadLocalRandom rand = ThreadLocalRandom.current();
        int []res = new int[nbCells];
        for(int i=0; i<nbCells; i++) res[i] = rand.nextInt(2);
        return res;
    }
}